﻿namespace TabsStudioMarker
{
    using TabExtensionList = System.Collections.Generic.List<string>;
    using TabList = System.Collections.Generic.List<
                        System.Collections.Generic.List<string>>;

    class SolutionStateSaver
    {
        public SolutionStateSaver(TabsStudioExt.ITabsStudioEngine engine_, EnvDTE80.DTE2 dte_)
        {
            engine = engine_;
            dte = dte_;
        }

        public void Save()
        {
            TabList tabList = new TabList();
            foreach(TabsStudioExt.Tabs tabs in engine.GetTabs())
            {
                foreach(TabsStudioExt.Tab tab in tabs.TabList)
                {
                    if (Properties.GetIsHighlighted(tab.TabItem))
                    {
                        TabExtensionList tabExtensionList = new TabExtensionList();
                        foreach (TabsStudioExt.TabExtension tabExtension in tab.TabExtensions)
                            tabExtensionList.Add(GetWindowID(tabExtension.Window));
                        tabList.Add(tabExtensionList);
                    }
                }
            }
            try
            {
                using (System.IO.FileStream stream =
                    new System.IO.FileStream(GetStoragePath(), System.IO.FileMode.Create))
                {
                    System.Xml.Serialization.XmlSerializer serializer =
                        new System.Xml.Serialization.XmlSerializer(typeof(TabList));
                    serializer.Serialize(stream, tabList);
                }
            }
            catch (System.Exception)
            {
            }
        }

        public void Load()
        {
            highlightedTabList = null;
            try
            {
                using (System.IO.FileStream stream =
                    new System.IO.FileStream(GetStoragePath(), System.IO.FileMode.Open,
                        System.IO.FileAccess.Read, System.IO.FileShare.Read))
                {
                    System.Xml.Serialization.XmlSerializer serializer =
                        new System.Xml.Serialization.XmlSerializer(typeof(TabList));
                    highlightedTabList = serializer.Deserialize(stream) as TabList;
                }
            }
            catch (System.Exception)
            {
            }
        }

        public void OnTabCreated(object sender, TabsStudioExt.TabEventArgs e)
        {
            if (highlightedTabList == null || highlightedTabList.Count == 0)
                return;

            string tabCreatedID = GetWindowID(e.Tab.TabExtensions[0].Window);
            if (FindAndRemoveHighlighted(tabCreatedID))
                Properties.SetIsHighlighted(e.Tab.TabItem, true);
        }

        private string GetStoragePath()
        {
            return System.IO.Path.ChangeExtension(dte.Solution.FullName, "marker.tss");
        }

        private bool FindAndRemoveHighlighted(string id)
        {
            foreach (TabExtensionList tabExtensionList in highlightedTabList)
            {
                foreach (string s in tabExtensionList)
                {
                    if (string.Compare(s, id, true) == 0)
                    {
                        highlightedTabList.Remove(tabExtensionList);
                        return true;
                    }
                }
            }
            return false;
        }

        private string GetWindowID(EnvDTE.Window window)
        {
	        try	
            { 
                if (window.Document != null) // can throw
                    return window.Document.FullName;
	        } 
            catch (System.Exception) 
            {
	        }
            return window.Caption;
        }

        private TabsStudioExt.ITabsStudioEngine engine;
        private TabList highlightedTabList;
        EnvDTE80.DTE2 dte;
    }
}
